@@ -84,7 +84,7 @@ class open:
     作用：1.主要是为了同时兼容py2和py3；2.增加了索引功能，方便读取大文件。
     """
     def __init__(
-        self, name, mode='r', encoding=None, errors=None, indexable=False
+        self, name, mode='r', encoding=None, errors='strict', indexable=False
     ):
         self.name = name
         if is_py2:
@@ -267,37 +267,46 @@ def parallel_apply(
         return [r[1] for r in results]
 
 
-def sequence_padding(inputs, length=None, padding=0, mode='post'):
+def sequence_padding(inputs, length=None, value=0, seq_dims=1, mode='post'):
     """Numpy函数，将序列padding到同一长度
     """
     if length is None:
-        length = max([len(x) for x in inputs])
+        length = np.max([np.shape(x)[:seq_dims] for x in inputs], axis=0)
+    elif not hasattr(length, '__getitem__'):
+        length = [length]
 
+    slices = [np.s_[:length[i]] for i in range(seq_dims)]
+    slices = tuple(slices) if len(slices) > 1 else slices[0]
     pad_width = [(0, 0) for _ in np.shape(inputs[0])]
+
     outputs = []
     for x in inputs:
-        x = x[:length]
-        if mode == 'post':
-            pad_width[0] = (0, length - len(x))
-        elif mode == 'pre':
-            pad_width[0] = (length - len(x), 0)
-        else:
-            raise ValueError('"mode" argument must be "post" or "pre".')
-        x = np.pad(x, pad_width, 'constant', constant_values=padding)
+        x = x[slices]
+        for i in range(seq_dims):
+            if mode == 'post':
+                pad_width[i] = (0, length[i] - np.shape(x)[i])
+            elif mode == 'pre':
+                pad_width[i] = (length[i] - np.shape(x)[i], 0)
+            else:
+                raise ValueError('"mode" argument must be "post" or "pre".')
+        x = np.pad(x, pad_width, 'constant', constant_values=value)
         outputs.append(x)
 
     return np.array(outputs)
 
 
-def truncate_sequences(maxlen, index, *sequences):
+def truncate_sequences(maxlen, indices, *sequences):
     """截断总长度至不超过maxlen
     """
     sequences = [s for s in sequences if s]
+    if not isinstance(indices, (list, tuple)):
+        indices = [indices] * len(sequences)
+
     while True:
         lengths = [len(s) for s in sequences]
         if sum(lengths) > maxlen:
             i = np.argmax(lengths)
-            sequences[i].pop(index)
+            sequences[i].pop(indices[i])
         else:
             return sequences
 
@@ -579,13 +588,14 @@ class AutoRegressiveDecoder(object):
             output_scores = np.take_along_axis(
                 scores, indices, axis=None
             )  # 更新得分
+            is_end = output_ids[:, -1] == self.end_id  # 标记是否以end标记结束
             end_counts = (output_ids == self.end_id).sum(1)  # 统计出现的end标记
             if output_ids.shape[1] >= self.minlen:  # 最短长度判断
-                best_one = output_scores.argmax()  # 得分最大的那个
-                if end_counts[best_one] == min_ends:  # 如果已经终止
-                    return output_ids[best_one]  # 直接输出
+                best = output_scores.argmax()  # 得分最大的那个
+                if is_end[best] and end_counts[best] >= min_ends:  # 如果已经终止
+                    return output_ids[best]  # 直接输出
                 else:  # 否则，只保留未完成部分
-                    flag = (end_counts < min_ends)  # 标记未完成序列
+                    flag = ~is_end | (end_counts < min_ends)  # 标记未完成序列
                     if not flag.all():  # 如果有已完成的
                         inputs = [i[flag] for i in inputs]  # 扔掉已完成序列
                         output_ids = output_ids[flag]  # 扔掉已完成序列
@@ -647,9 +657,10 @@ class AutoRegressiveDecoder(object):
                     k_indices, sample_ids, axis=1
                 )  # 对齐原id
             output_ids = np.concatenate([output_ids, sample_ids], 1)  # 更新输出
+            is_end = output_ids[:, -1] == self.end_id  # 标记是否以end标记结束
             end_counts = (output_ids == self.end_id).sum(1)  # 统计出现的end标记
             if output_ids.shape[1] >= self.minlen:  # 最短长度判断
-                flag = (end_counts == min_ends)  # 标记已完成序列
+                flag = is_end & (end_counts >= min_ends)  # 标记已完成序列
                 if flag.any():  # 如果有已完成的
                     for ids in output_ids[flag]:  # 存好已完成序列
                         results.append(ids)
@@ -745,6 +756,24 @@ def longest_common_subsequence(source, target):
     return l, mapping[::-1]
 
 
+def orthogonally_resize(a, new_shape, window=2):
+    """简单的正交化缩放矩阵
+    """
+    assert a.ndim == len(new_shape)
+    slices, a_norm, w = [], np.linalg.norm(a), window
+    for i, (d1, d2) in enumerate(zip(a.shape, new_shape)):
+        if d1 != d2:
+            k = d2 // d1 + int(d2 % d1 != 0)
+            if k > 1:
+                assert d1 % w == 0
+                a = a.reshape(a.shape[:i] + (d1 // w, w) + a.shape[i + 1:])
+                a = np.repeat(a, k, axis=i)
+                a = a.reshape(a.shape[:i] + (d1 * k,) + a.shape[i + 2:])
+        slices.append(np.s_[:d2])
+    a = a[tuple(slices)]
+    return a / np.linalg.norm(a) * a_norm
+
+
 class WebServing(object):
     """简单的Web接口
     用法：
@@ -831,7 +860,7 @@ class Hook:
         self.module = module
 
     def __getattr__(self, attr):
-        """使得 from bert4keras.backend import uniout
+        """使得 from bert4keras.snippets import uniout
         等效于 import uniout （自动识别Python版本，Python3
         下则无操作。）
         """
