@@ -26,19 +26,9 @@ def load_vocab(dict_path, encoding='utf-8', simplified=False, startswith=None):
             keep_tokens.append(token_dict[t])
 
         for t, _ in sorted(token_dict.items(), key=lambda s: s[1]):
-            if t not in new_token_dict:
-                keep = True
-                if len(t) > 1:
-                    for c in Tokenizer.stem(t):
-                        if (
-                            Tokenizer._is_cjk_character(c) or
-                            Tokenizer._is_punctuation(c)
-                        ):
-                            keep = False
-                            break
-                if keep:
-                    new_token_dict[t] = len(new_token_dict)
-                    keep_tokens.append(token_dict[t])
+            if t not in new_token_dict and not Tokenizer._is_redundant(t):
+                new_token_dict[t] = len(new_token_dict)
+                keep_tokens.append(token_dict[t])
 
         return new_token_dict, keep_tokens
     else:
@@ -111,7 +101,12 @@ class TokenizerBase(object):
         return [self.token_to_id(token) for token in tokens]
 
     def encode(
-        self, first_text, second_text=None, maxlen=None, pattern='S*E*E'
+        self,
+        first_text,
+        second_text=None,
+        maxlen=None,
+        pattern='S*E*E',
+        truncate_from='right'
     ):
         """输出文本对应token id和segment id
         """
@@ -123,22 +118,28 @@ class TokenizerBase(object):
         if second_text is None:
             second_tokens = None
         elif is_string(second_text):
-            if pattern == 'S*E*E':
-                idx = int(bool(self._token_start))
-                second_tokens = self.tokenize(second_text)[idx:]
-            elif pattern == 'S*ES*E':
-                second_tokens = self.tokenize(second_text)
+            second_tokens = self.tokenize(second_text)
         else:
             second_tokens = second_text
 
         if maxlen is not None:
-            index = int(self._token_end is not None) + 1
-            truncate_sequences(maxlen, -index, first_tokens, second_tokens)
+            if truncate_from == 'right':
+                index = -int(self._token_end is not None) - 1
+            elif truncate_from == 'left':
+                index = int(self._token_start is not None)
+            else:
+                index = truncate_from
+            if second_text is not None and pattern == 'S*E*E':
+                maxlen += 1
+            truncate_sequences(maxlen, index, first_tokens, second_tokens)
 
         first_token_ids = self.tokens_to_ids(first_tokens)
         first_segment_ids = [0] * len(first_token_ids)
 
         if second_text is not None:
+            if pattern == 'S*E*E':
+                idx = int(bool(self._token_start))
+                second_tokens = second_tokens[idx:]
             second_token_ids = self.tokens_to_ids(second_tokens)
             second_segment_ids = [1] * len(second_token_ids)
             first_token_ids.extend(second_token_ids)
@@ -355,6 +356,19 @@ class Tokenizer(TokenizerBase):
         """
         return bool(ch) and (ch[0] == '[') and (ch[-1] == ']')
 
+    @staticmethod
+    def _is_redundant(token):
+        """判断该token是否冗余（默认情况下不可能分出来）
+        """
+        if len(token) > 1:
+            for ch in Tokenizer.stem(token):
+                if (
+                    Tokenizer._is_cjk_character(ch) or
+                    Tokenizer._is_punctuation(ch)
+                ):
+                    return True
+
+
     def rematch(self, text, tokens):
         """给出原始的text和tokenize后的tokens的映射关系
         """
